;+
; NAME:
;	 hxt_multimg
; PURPOSE:
;	 To make HXT images.
; CATEGORY:
;        Yohkoh HXT
; CALLING SEQUENCE:
;        Hxt_multimg, index, data, index_out, data_out, patterns, $
; GENERAL KEYWORDS:
;            outfile = outfile, /append_outfile, channels = channels, $
;            /Nolook, /nonorm, /bsmooth, Sys_err = sys_err, /prompt_outfile, $
;            error=error, /calc_btot, amp_err = amp_err, $
;            phase_err = phase_err, $
; ACCUMULATION KEYWORDS:
;            img_trange = img_trange, bck_trange = bck_trange, /alt_tsel, $
;            dt_resolution = dt_resolution, accum_cnts = accum_cnts, $
;            ch_accum = ch_accum, /noback, st_times = st_times, $
;            int_times = int_times $
; MODULATION PATTERN KEYWORDS:
;            /new_mod_patterns, xy = xy, expfac = expfac, ntrn = ntrn, $
;            n_pix = n_pix, patt_file = patt_file, grause_file = grause_file, $
; MEM KEYWORDS:
;            gamma0 = gamma0, chilim = chilim, Lambda_limit = lambda_limit, $
;            lambda_max=lambda_max, delta_max=delta_max, iter_max=iter_max, $
; QLOOK KEYWORDS:
;            /qlook, bin = bin, smooth = smooth, $
; PIXON KEYWORDS:
;            /pixons, /snr, guess = guess, /quiet, $
;            /verbose, /continued, /partan, $
;            resolution = resolution, pixon_sizes = pixon_sizes, $
;            /notty, /lo2hi_guess, /poisson, /chisqr, sensitivity, $
;            pixonmap=pixonmap
;INPUT:
;	index = HXT data info from hda file
;	data = counts in each subcollimator for each channel
;       patterns = the modulation patterns, (optional), if not passed in,
;                 or if any of the mod patterns keywords
;                 have been set, then new patterns will be calculated.
;KEYWORD INPUT:
; GENERAL KEYWORDS:
;       outfile = an output filename, NOTE: this will be made lower case.
;                 Unlike previous versions, HXT_MULTIMG no longer
;                 prompts for an output file, this must be set if you
;                 want to output into a file.
;       /append_outfile = if set, and the output file already exists,
;                        append the output file
;       /prompt_outfile = if set, the routine prompts you for an output
;                         filename after the images have been generated,
;                         THIS IS IGNORED IF YOU'VE SET THE OUTFILE KEYWORD...
;       channels = if set, will only image the channels in channels,
;                  otherwise all channels are imaged.
;       /nolook = do not plot the intermediate images,
;       /nonorm = do not normalize images, the default is to normalize
;                the images to byte arrays of 0 to 255, using the routine
;                HXI_NORMAP. 
;       /bsmooth= if set, smooth the image over 3 pixels, passed into
;                 MAKIMG, AND MAKIMG_L. This is here to avoid the
;                 overresolution COMMON in  MEM images, but you may use this
;                 for the pixon case too.
;       /chisqr= if set, sigma(counts) is a weighted sum of total(sqrt(counts))
;                 and sqrt(total(counts)), where the total is taken over the
;                 accumulation intervals, This was the default, but is not
;                 any longer, 6-mar-96, jmm
;       error = returns with an error estimate for the HXT images.  Suitable
;               for use with hxt_lcurve.  CAUTION: These are statistical 
;               errors only.  There is no account for reconstruction errors.
;               Returned with MEM and Pixon reconstructions only.
;       calc_btot = if set, uses HXT_BTOT to calculate the total counts in the
;                   image, 
; ACCUMULATION KEYWORDS:
;       img_trange = a time range for the images, any Yohkoh time format...
;       bck_range = a time_range for the background accumulation
;       /alt_tsel = if set, use alt_tim_sel to get interval, instead of
;                  plot_lcur, this is good if you have no xwindows.
;       dt_resolution = number of 1/2 second (flare mode) or 4 second
;                      (quiet mode) frames per image, default is to
;                      use the whole time range. NOTE THAT THIS KEYWORD IS
;                      IGNORED IF MULTIPLE INTERVALS HAVE BEEN SELECTED
;                      EITHER IN PLOT_LCUR, OR IF THEY HAVE BEEN PASSED IN.
;       accum_cnts = if set, the value of average counts/SC in a given
;                    that you want for each set of images, the routine
;                    will accumulate counts until the number of counts
;                    in the given channel are greater than or equal to
;                    this value. If set with a slash, ie. the value=1
;                    then the default is 200 (a good value).
;                    As of 9-jun-95, this refers to EXCESS counts, as
;                    opposed to total counts. This keyword will also
;                    not work if multiple intervals are selected while
;                    in PLOT_LCUR, or are passed in.
;       ch_accum = the channel you want to use for the accum_cnts (future
;                  versions will allow you to accum different numbers of
;                  counts for different channels).
;       /noback = if set, the background will be set to zero, this is
;                not recommended for MEM images.
;	sys_err= systematic error term to be added to Poisson Uncertainty,
;                default is 0.01, as of jan-1997
;                As of june-1998, sys_err is only used for PIXON images
;       amp_err = systematic amplitude error term, either a scalar, or
;                 an array of nchannels, one value for each channel
;                 used, or an array of four, corresponding to all four
;                 channels (the program will pick out the appropriate
;                 value for each channel).
;                 the default is [0.01, 0.015, 0.02, 0.03]
;                 for the four channels
;	phase_err= systematic phase error term, the default is 0.315
;       st_times= an array of accumulation start times, in any yohkoh format
;       int_times= interval times corresponding to st_times, may be a scalar,
;                  if this is not set, and st_times is set, the routine will
;                  use the differences between st_times. This must be an integer
;                  number of the appropriate resolution in seconds, if the
;                  value of int_times is less than a frame value.
; MODULATION PATTERN KEYWORDS:
;       /new_mod_patterns = If set, Force a new computation of the
;                         modulation patterns, even if they exist already, 
;                         even if the patterns parameter is set.  Normally, if
;                         patterns is set, the modulation patterns are not
;                         computed.  You must recompute the modulation 
;                         patterns whenever you use a different pointing.
;                         If any of the mod. pattern keywords are set,
;                         the mod. patterns are recomputed.
;       xy= an array containing the 2 HXA aspect coordinates, if not
;           set, and the patterns are to be calculated, the program will prompt
;           you. This now (as of 18-oct-1996) allows you
;           to pass in an sxt_index as the corrdinates, or a string
;           representation of the position, e.g. 'n16w13'
;       expfac= expansion factor, the default is 1.0, gives 2.47 arcsec pixels.
;               The routine will no longer prompt you.
;       n_pix= number of pixels in image, default is 64, the default n_pix and
;              expfac give the same size as a Full res. flare mode PFI from SXT.
;       patt_file = a filename for the mod. pattern parameters, the default
;                      is $DIR_HXT_CAL/npara.dat, as of 13-may-1998
;       grause_file = a filename for the grid use file, the default is
;                     $DIR_HXT_CAL/grause, or an array of 64 1's or 0's
;                     if set to 0, that subcollimator is not used
;       ntrn = the number of points in the 1-d mod patterns for each SC
;              the default is 512
; MEM METHOD KEYWORDS:
;       gamma0= initial value of iteration gain, default is 0.08
;	chilim= minimum value of chi^2 needed in MEM iteration, default is 2.0
;       lambda_limit = if set, makimg will iterate to the given value
;                      of lambda.
;       lambda_max = max value of lambda allowed, default is 50, note that
;                    this is different than lambda_limit, use of lambda_limit
;                    turns off the Chi^2 criterion, use of lambda_max keeps
;                    the Chi^2 criterion.
;       iter_max = max no. of iterations for a given lambda, default is 150
;       delta_max = max value of delta for a switch to a new lambda, default is 0.03
; PIXON KEYWORDS:
;       /pixons= if set uses Tom Metcalf;s HXTPIX to get the images, in pixons.
;       guess = initial guess for reconstructed image (def = compute with
;              hxt_qlook).  If guess is not n_pix by n_pix or if all elements
;              of guess are LE 0, then the guess is not used.  If you pass
;              a standard image (i.e. north up east left) the image should
;              be flipped before passing as a guess: guess = rotate(guess,7).
;              Guess can be a vector of images provided that the dimensions
;              of guess are the same as the dimensions of data_out.  HOWEVER,
;              this will only work if you analyze a single energy channel, i.e.
;              you must pass something like chan=[0].
;       /lo2hi_guess = bootstrap the initial guesses, i.e use the LO image as 
;              a guess for the M1 image, the M1 as a guess for the M2, and
;              the M2 as a guess for the HI.  To make
;              this work, be sure to specify channels from LO up to whatever
;              channel you are interested in, e.g. chan=[0,1,2],/lo2hi_guess.
;       /snr = Weight the residuals by the signal-to-noise ratio.
;       resolution = smallest allowed pixon size (pixels), default = 1
;       pixon_sizes = an integer list of pixon sizes (resolutions) to use.
;       /partan = Uses "parallel tangent" iteration.  Slower than the default
;               conjugate gradient, but almost always converges even when
;               conjugate gradient fails.
;       /continued = Uses faster "continued parallel tangent"
;                  iteration scheme.
;                  It is slower than conjugate gradient, but faster than
;                  plain parallel tangent
;       /notty = Not running on a tty.  Set this for background jobs.
;       /quiet = Don't print any diagnostics
;       /verbose = print lots of diagnostics
;       /poisson, /chisqr = If /pixons is set, the /poisson and /chisqr
;                 keywords have the following effects: 
;                  -----------------------
;                 poisson = 0, chisqr = 0:
;                 If neither /poisson nor /chisqr are set, the error
;                 is set to sqrt(total(counts)), as in MEM. This is the
;                 default behavior which should be appropriate for most cases. 
;                 ------------------------
;                 chisqr = 1:
;                 If /chisqr is set, regardless of whether /poisson is set,
;                 the error is set to a weighted average of sqrt(total(counts))
;                 and total(sqrt(counts)).  This is useful when the count
;                 rate is low or when the integration time is long.  
;                 -----------------------
;                 poisson = 1, chisqr = 1:
;                 If /poisson is set and /chisqr is not set, the error
;                 computed by hxtimg_accum is ignored in the pixon code which
;                 uses Poisson statistics based on the counts above
;                 background. This behavior may not be what you expect.
;       pixonmap = returns with the pixon maps
;       sensitivity = controls the sensitivity of the pixon map.  0.0 is
;                          most sensitive.  Def = sqrt(2.).
;OUTPUT:
;       index_out = index for images, this contains the HDA index for
;                   the start of each image accumulation time,
;                   The tag index_out.hxi contains the structure hxi_index,
;                   with tags:
;	(DATA SELECTION)
;	hxi_index.chan= channel used
;		=0 L0 (13.7-22.7 keV)
;		=1 M1 (22.7-32.7 keV)
;		=2 M2 (32.7-52.7 keV)
;		=3 HI (52.7-92.8 keV)
;	cnts_p_cm2= counts per cm^2 for the selected channel (not per sec)
;                   for the 16 fan beam collimators*some normalization factor
;	hxi_index.actim= image accumulation time in tenths of seconds.
;	hxi_index.interval= end time minus start time =actim if there's
;                           no data missing.
;	hxi_index.bkg_time= start time for background accumulation
;                           (millisec of day)
;	hxi_index.bkg_day= day of start of background accumulation
;                          (days since 1-Jan-79)
;	hxi_index.bkg_actim= background accumulation time in millisec 
;	hxi_index.bkg_interval= end time minus start time
;	(INPUT PARAMETERS TO THE SYNTHESIS ROUTINES)
;	hxi_index.grause(8)=Each element is from 0b to 255b, 
;		If hxi_index.grause(j)=0b, then the Subcollimators with 
;		spatial wave number j+1 aren't used in the calculation
;	hxi_index.x0, hxi_index.y0= X and Y corrdinates in HXT Coordinate
;               system,  can be obtained frm SXT data using
;               conv_p2hxt(gt_center(sxt_index)), may be input interactively.
;	hxi_index.alpha= amplitude of phase correction factor = -1.0
;	hxi_index.iway=0 for triangular patterns, =1 for sine wave patterns
;	hxi_index.area= total area in cm^2 of scintillators
;	hxi_index.syserr= systematic error term to be added to Poisson
;                         Uncertainty
;	hxi_index.gamma0= initial value of iteration gain (NOT USED FOR PIXONS)
;	hxi_index.laminc= initial value for lambda increment (NOT USED FOR PIXONS)
;	hxi_index.chilim= minimum value of chi^2 needed (NOT USED FOR PIXONS)
;       hxi_index.perr = phase error term
;       hxi_index.aerr = amplitude error term
;	(RESULTS OF SYNTHESIS ROUTINES)
;	hxi_index.lambda= final value of lambda, which controls the 
;                         iteration step size. (NOT USED FOR PIXONS)
;	hxi_index.delta= total((b(i)-b(i-1))^2)/Total(b(i)^2) for each
;                        iteration, lambda is changed when delta is reduced
;                        to 0.030 or less.  (NOT USED FOR PIXONS)
;	hxi_index.iteration= the number of iterations, for the final value of
;                            lambda.
;	hxi_index.chi2= chi^2, reduced = total((fobs(b(i))-fobs)^2/var)/usenum
;		where usenum is the number of SC's used
;	hxi_index.max_bright= maximum brightness of image
;	hxi_index.x_max= Column of max. brightness
;	hxi_index.y_max= row of max. brightness
; And the answer:
;       data_out = data for images, in counts/cm^2/SC,  not per second,
;                  normalized from 0 to 255, unless the /nonorm keyword is set,
;                  normalized data can be unnormalized using the routine HXI_UNNORM.
;DESCRIPTION:
;       Index and Data come from an HDA file, The position in HXA coordinates
;       Can be obtained in various ways, using GET_HXT_POS, or from SXT images
;       using xy = gt_center(sxt_index, /hxt). You choose the time ranges,
;       as a default. Once counts are accumulated, the images are synthesized
;       using the Max Entropy method (default), the Pixon method (if you've
;       set the /pixons keyword), or zeroth order regularization (/qlook).
;       The image is normalized before being passed out of the routine.
;MODIFICATION HISTORY:
;	Adapted from program movie7 of T. Sakao, by J.McTiernan, June '92
;       Lots o' revisions, 18-jul-94, jmm
;       different mod patterns for different channels, 20-jul-94, jmm
;       account for 4 second time lag, 21-sep-94, jmm
;       Added xxx keyword, 14-feb-95, jmm
;       redefined accum_cnts keyword to excess counts, not total counts,
;              9-jun-95, jmm
;       Got sick of all of these keywords, got rid of pixon and qlook options,
;              stick with MEM, changed defaults for output file, 13-jun-95, jmm
;       Oops, Pixons are back, reorganized keyword list,
;              Hopefully things are understandable, added modifications
;              of Tom Metcalf for pixon code, and batch running, 22-Jun-95, jmm
;       Put the prompt for output files back in, as a keyword, jmm 11-jul-95
;       Added lambda_max, delta_max, iter_max keywords, 1-aug-95, jmm
;       Put back in restriction of fobs>0.0, which was removed when GETVAR
;       was retired, 22-sep-95, jmm
;       Changed  default sigmas back to poisson statistics, added /poisson
;       and /chisqr keywords, 6-mar-96, jmm
;       T. Metcalf 1996-10-01 Added calls to hxt_error.  Added pixon keywords
;                  sensitivity, pixonmap.  Added error, imagedata, imagesig
;                  keywords.
;       T. Metcalf 1996-10-10 Fixed a bug in the pixon error analysis which
;                  did not return the error estimate properly when multiple
;                  channels were analyzed.
;       Added the option to pass in an SXT index, or string for the flare
;       position, jmm, 18-oct-1996.
;       Added patt_file and grause_file keywords, jmm, 26-nov-96
;       Fixed bug in which the chilim keyword was not being passed into
;       MAKIMG, jmm, 3-jan-97
;       Added st_times, int_times keywords, jmm, 3-jan-97
;       sato_patterns, added, 14-jan-1997
;       added call to HXT_BTOT, by Tom Metcalf, for the MEM images
;       27-jan-1997, jmm, this can be turned on by setting /calc_btot
;       new mod. patterns, jmm, 4-jun-1998, sato_patterns is no longer
;       a valid keyword
;-
PRO Hxt_multimg, index, data, index_out, data_out, p, Outfile=outfile, $
                 prompt_outfile=prompt_outfile, $
                 append_outfile=append_outfile, Channels=channels, $
                 Nolook=nolook, Nonorm=nonorm, Bsmooth=bsmooth, $
                 poisson=poisson, Img_trange=img_trange, $
                 Bck_trange=bck_trange, alt_tsel=alt_tsel, $
                 Dt_resolution=dt_resolution, Accum_cnts=accum_cnts, $
                 Ch_accum=ch_accum, Noback=noback, Sys_err=sys_err, $
                 New_mod_patterns=new_mod_patterns, Xy=xy, Expfac=expfac, $
                 N_pix=n_pix, Gamma0=gamma0, Chilim=chilim, $
                 Lambda_limit=lambda_limit, lambda_max=lambda_max, $
                 delta_max=delta_max, iter_max=iter_max, $
                 Qlook=qlook, Bin=bin, Smooth=smooth, calc_btot=calc_btot, $
                 Pixons=pixons, Snr=snr, Guess=guess, Quiet=quiet, $
                 Verbose=verbose, Continued=continued, Partan=partan, $
                 Resolution=resolution, Pixon_sizes=pixon_sizes, $
                 Notty=notty, Lo2hi_guess=lo2hi_guess, chisqr=chisqr, $
                 error=error, pixonmap=pixonmap, imagedata=imagedata, $
                 imagesig=imagesig, sensitivity=sensitivity, $
                 patt_file=patt_file, grause_file=grause_file, $
                 St_times=st_times, Int_times=int_times, $
                 sato_patterns=sato_patterns, Amp_err=amp_err, $
                 Phase_err=phase_err, Ntrn=ntrn
;
   COMMON Mod_patterns, sc_paras, ictrl, hxi_index0
   
   print, '***HI, HXT_MULTIMG IS NEW AS OF 25-JUN-1998 '
   print, '***NEW IMPROVED SATO PATTERNS, AND MEM ALGORITHMS ARE USED '
   print, '***TRY doc_library, "hxt_multimg"'

   IF(KEYWORD_SET(sato_patterns)) THEN BEGIN
      message, 'The original Sato patterns have been dis-implemented', /info
      print, 'The default patterns are the improved Sato patterns'
   ENDIF
   
   grdnum = 64 
   ch_ch = [' LO ',' M1 ',' M2 ',' HI ']
; For the variance for the SC data, new as of 5-jun-1998
   hgain_arr = [11.1012, 11.1012, 11.1177, 11.6890]
; The integer wave number for each collimator, used in the variance
; calculation
   kpi = fltarr(64)+1
   FOR jjj = 2, 7 DO kpi(8*jjj:8*(jjj+1)-1) = jjj+1
; Do we calculate new mod. patterns??
   IF(N_ELEMENTS(p) EQ 0) THEN yn = 0 ELSE yn = 1
   IF(KEYWORD_SET(new_mod_patterns)) THEN yn = 0
   IF(KEYWORD_SET(xy)) THEN BEGIN
      yn = 0
      xy_p = xy
   ENDIF ELSE xy_p = 0
   IF(KEYWORD_SET(expfac)) THEN BEGIN
      yn = 0
      expf = expfac
   ENDIF ELSE expf = 0
   IF(KEYWORD_SET(n_pix)) THEN BEGIN
      yn = 0
      pxlnum = n_pix
   ENDIF ELSE pxlnum = 64
   IF(KEYWORD_SET(patt_shape)) THEN BEGIN
      yn = 0
      pshape = patt_shape
   ENDIF ELSE pshape = 0
   IF(KEYWORD_SET(ph_correct)) THEN BEGIN
      yn = 0
      phc = ph_correct
   ENDIF ELSE phc = 0
   IF(KEYWORD_SET(patt_file)) THEN BEGIN
      yn = 0
      pattfile = patt_file
   ENDIF ELSE pattfile = 0
   IF(KEYWORD_SET(grause_file)) THEN BEGIN
      yn = 0
      grfile = grause_file
   ENDIF ELSE grfile = 0
   
   IF(yn EQ 0) THEN BEGIN
      initpatt = 1
; Define Structure, and add some defaults, no more Patt_default.pro
      hxt_struct, hxi_index = hxi_index0
      hxi_index0.alpha = -1.0
      hxi_index0.iway = 0
      hxi_index0.area = 2.3*2.3-0.25*0.25*2.0 ;14-jan-1997
      hxi_index0.grause(*) = 255b
      hxt_pattern_maker, hxi_index0, p, sc_paras, xy = xy_p, $
        expfac = expf, n_pix = pxlnum, date = index(0), patt_file = pattfile, $
        grause_file = grfile, ntrn = ntrn
   ENDIF ELSE BEGIN
      initpatt = 0
      pxlnum = N_ELEMENTS(p(*, 0, 0))
      hxi_index0.shape_sav(*) = pxlnum ;gotta have it, jmm 3-jan-1997
   ENDELSE
   area = hxi_index0.area       ;makes the code look nicer later
   pix2 = pxlnum^2
   
;parameters for the image synthesis, img_default is gone. jmm, 4-jun-1998
   IF(N_ELEMENTS(gamma0) gt 0) THEN hxi_index0.gamma0 = gamma0(0) $
     ELSE hxi_index0.gamma0 = 0.08
   IF(N_ELEMENTS(chilim) GT 0) THEN hxi_index0.chilim = chilim(0) $
     ELSE hxi_index0.chilim = 1.0
   hxi_index0.laminc = 1
;Sys_err is retained for the pixons, as of now, jmm, 4-jun-1998
   IF(KEYWORD_SET(pixons)) THEN BEGIN
      IF(N_ELEMENTS(sys_err) GT 0) THEN hxi_index0.syserr = sys_err(0) $
        ELSE hxi_index0.syserr = 0.01
   ENDIF ELSE hxi_index0.syserr = 0.0
   IF(N_ELEMENTS(phase_err) GT 0) THEN hxi_index0.perr = phase_err(0) $
     ELSE hxi_index0.perr(0) = 0.315

;The amp_error is tricky first get the channels used
   IF(N_ELEMENTS(channels) GT 0) THEN chans = channels $
     ELSE chans = [0, 1, 2, 3]
   nch = N_ELEMENTS(chans)
   naerr = N_ELEMENTS(amp_err)
   IF(Naerr GT 0) THEN BEGIN
;For a scalar, all of these will be the same
      IF(naerr EQ 1) THEN aerr_arr = replicate(amp_err(0), nch) ELSE BEGIN
;If amp_err has the same number of elements as chans, then it's easy
         IF(naerr EQ nch) THEN aerr_arr = amp_err ELSE BEGIN
;otherwise, there must be 4 elements in amp_err, one for each possible channel
            IF(naerr EQ 4) THEN aerr_arr = amp_err(chans) ELSE BEGIN
               message, /info, 'Mismatch of amp_err and channels, returning'
               RETURN
            ENDELSE
         ENDELSE
      ENDELSE
   ENDIF ELSE BEGIN
      aerr0 = [0.01, 0.015, 0.02, 0.03]
      aerr_arr = aerr0(chans)
   ENDELSE
   IF(KEYWORD_SET(nolook) OR KEYWORD_SET(notty)) THEN ictrl = 0 ELSE ictrl = 1
   
; You need an initial version of hxi_index to pass into the accumulation
; routine, hxi_index0 is saved in a common block
   hxi_index = hxi_index0
; get the time intervals for background and image, and accumulate counts
   IF(nch EQ 1) THEN ch_to_plot = [chans(0)] ELSE ch_to_plot = 0
   Hxtimg_accum, index, data, index1, hxi_index, counts, sigma, bcounts, $
     bsigma, img_trange = img_trange, bck_trange = bck_trange, $
     dt_resolution = dt_resolution, alt_tsel = alt_tsel, $
     accum_cnts = accum_cnts, ch_accum = ch_accum, noback = noback, $
     chisqr = chisqr, channel = ch_to_plot, st_times = st_times, $
     int_times = int_times
   
; subtract background
   IF(KEYWORD_SET(noback)) THEN zb = 1 ELSE zb = 0 ;probably unneccesary, jmm, 6/22/95
   hxtsbt_mult, hxi_index, counts, bcounts, fobs, sig, b_zero = zb, $
     csigma = sigma, bsigma = bsigma
   
; make the images, for each channel and interval
   n_intv = N_ELEMENTS(index1)
   nch1 = nch-1
   hxi_index = hxi_index(chans, *)
; Assign the appropriate amplitude error to hxi_index here, yuck
   FOR j = 0, nch-1 DO BEGIN
      iii = where(hxi_index.chan EQ chans(j))
      hxi_index(iii).aerr = aerr_arr(j)
   ENDFOR
   fobs = fobs(*, chans, *) & sig = sig(*, chans, *)
   fobs = fobs > 0.0            ;jmm 22-sep-95, lost when GETVAR was removed
   sig = sig > 1.0              ;jmm 22-sep-95, lost when GETVAR was removed
   
   ndset = n_intv*nch
;If btot is to be calculated, do it here
   IF(KEYWORD_SET(calc_btot)) THEN BEGIN
      print, 'calculating btot'
      btot0 = hxt_btot(fobs, p(*, *, *, chans))
   ENDIF
   
; Do we output the data? New stuff, we'll output only if the
; outfile keyword is set, and append only if the append
; keyword is set. 22-Jun-95, jmm
   IF(KEYWORD_SET(outfile)) THEN BEGIN
      outfil = outfile
      append = 0
   ENDIF ELSE IF(KEYWORD_SET(prompt_outfile)) THEN BEGIN
;prompt for filename
      exten_wvq = time_2_fileid(index1(0), /sec)
      def_filnam = 'hxi' + exten_wvq
      input, 'ENTER THE OUTPUT FILE NAME', outfil, def_filnam
      append = 0
   ENDIF ELSE outfil = 'none'
   
   outfil = strlowcase(outfil)
   IF (file_exist(outfil)) THEN BEGIN
      print, 'FILE ', outfil, ' EXISTS'
      IF(KEYWORD_SET(append_outfile)) THEN BEGIN
         print, 'IT WILL BE APPENDED'
         append = 1
      ENDIF ELSE BEGIN
         print, 'IT WILL BE OVERWRITTEN, TO APPEND SET /APPEND_OUTFILE'
         append = 0
      ENDELSE
   ENDIF

;output arrays
   data_out = fltarr(pxlnum, pxlnum, ndset)
   error = fltarr(pxlnum, pxlnum, ndset)
   index_out = replicate(str_merge(index1(0), hxi_index(0)), ndset)
   imagedata = fltarr(grdnum,ndset)
   imagesig = fltarr(grdnum,ndset)
;Pixons and qlook are back, jmm 6/22/95
   IF(KEYWORD_SET(qlook)) THEN BEGIN
      IF(KEYWORD_SET(smooth)) THEN alpha = smooth ELSE alpha = 0.10
      IF(KEYWORD_SET(bin)) THEN bnn = bin ELSE bnn = 3
      FOR j = nch1, 0, -1 DO BEGIN
         print, ch_ch(chans(j)), 'CHANNEL '
         makimg0, ictrl, reform(hxi_index(j, *)), index1, reform(fobs(*, j, *)), $
           reform(sig(*, j, *)), p(*, *, *, chans(j)), index_j, data_j, $
           alpha, bin = bnn, quiet = quiet, verbose = verbose
         FOR k = 0, n_intv-1 DO BEGIN
            dset = k*nch+j
            str_load, index_out, index = dset, index_j(k)
            data_out(*, *, dset) = data_j(*, *, k)
            imagedata(*,dset) = reform(fobs(*, j, k))
            imagesig(*,dset) = reform(sig(*, j, k))
         ENDFOR
      ENDFOR
; hxt_error does not work well with quick look data since the image
; is so crude.
;;message,/info,'Computing error estimate'
;;error=hxt_error(index_out,imagedata,p,data_out,imagesig)
      IF(KEYWORD_SET(bsmooth)) THEN BEGIN
         qsmooth = 3
         FOR j = 0, ndset-1 DO $
           data_out(*, *, j) = smooth(data_out(*, *, j), qsmooth)
      ENDIF
   ENDIF ELSE IF(KEYWORD_SET(pixons)) THEN BEGIN
; The following tags are irrelevant for the pixon image reconstruction
      hxi_index(*).lambda = 0
      hxi_index(*).chilim = 0
      hxi_index(*).laminc = 0
      hxi_index(*).gamma0 = 0
      hxi_index(*).delta = 0
; This is still a mess...
      pixonmap = fltarr(pxlnum, pxlnum, ndset)
      IF(nch GT 1) THEN chanorder = sort(chans) $ ; go from lo to hi energy
        ELSE chanorder = [0]    ;sort doesn't work on scalars, jmm 22-jun-95

; Start imaging
      FOR jch = 0, nch1 DO BEGIN
         j = chanorder(jch)
; You need the variance for the SC data, new as of 15-jan-97,
; Newer as of 5-jun-1998, jmm
         hgain = hgain_arr(chans(j))
         FOR k = 0, n_intv-1 DO BEGIN
            btjk = total(fobs(*, j, k))/(area*hgain) ;an estimate of btot
            perrjk = hxi_index(j, k).perr
            aerrjk = hxi_index(j, k).aerr
            hxi_index(j, k).cnts_p_cm2 = btjk
;this is different for fan-beams and sin-cos collimators, sigh           
            sig(0:15, j, k) = sqrt(sig(0:15, j, k)^2+ $
                                   (0.25*area*btjk*aerrjk)^2+ $
                                   (perrjk*area*btjk/63.0)^2)
            sig(16:*, j, k) = sqrt(sig(16:*, j, k)^2+ $
                                   (0.5*area*btjk*aerrjk)^2+ $
                                   (perrjk*kpi(16:*)*area*btjk/126.0)^2)
         ENDFOR

         print, ch_ch(chans(j)), 'CHANNEL '
         IF KEYWORD_SET(lo2hi_guess) AND jch GT 0 THEN BEGIN
            guess = last_pixon_image
         ENDIF
         IF(KEYWORD_SET(calc_btot)) THEN btot_j = reform(btot0(j, *)) ELSE btot_j = 0
         hxtpix, ictrl, reform(hxi_index(j, *)), index1, reform(fobs(*, j, *)), $
           reform(sig(*, j, *)), p, index_j, data_j, quiet = quiet, verbose = verbose, $
           initpatt = initpatt, continued = continued, partan = partan, $
           resolution = resolution, snr = snr, guess = guess, $
           n_pix = long(pxlnum), pixon_sizes = pixon_sizes, notty = notty, $
           poisson = poisson, chisqr = chisqr, pixonerr=error_j , $
           pixonmap = pixonmap_j, sensitivity = sensitivity, btot_in = btot_j    
         IF KEYWORD_SET(lo2hi_guess) AND (jch LT nch1) THEN last_pixon_image = data_j
         FOR k = 0, n_intv-1 DO BEGIN
            IF KEYWORD_SET(lo2hi_guess) AND (jch LT nch1) THEN BEGIN
; Flip N/S and renormalize to the next filter.  The normalization is only approximate.
               last_pixon_image(*, *, k) = rotate(last_pixon_image(*, *, k), 7)* $
                 total(fobs(*, chanorder(jch+1), k))/total(fobs(*, j, k))
            ENDIF
            dset = k*nch+j
            str_load, index_out, index = dset, index_j(k)
            data_out(*, *, dset) = data_j(*, *, k)
            error(*, *, dset) = error_j(*, *, k)
            pixonmap(*, *, dset) = pixonmap_j(*, *, k)
         ENDFOR
      ENDFOR
      IF(KEYWORD_SET(bsmooth)) THEN BEGIN
         qsmooth = 3
         FOR j = 0, ndset-1 DO data_out(*, *, j) = smooth(data_out(*, *, j), qsmooth)
      ENDIF
   ENDIF ELSE BEGIN             ;MEM is the default
;for each channel, create images for each interval
      FOR j = nch1, 0, -1 DO BEGIN
         ppp = reform(transpose(p(*, *, *, chans(j)), [2, 0, 1]), grdnum, pix2)
         print, ch_ch(chans(j)), 'CHANNEL '
         FOR k = 0, n_intv-1 DO BEGIN
            index1_k = index1(k)
            print, 'INTERVAL START TIME:', fmt_tim(index1_k, /msec)
            print, 'ACCUMULATION TIME:', hxi_index(0, k).actim/10.0
            fobs_jk = fobs(*, j, k) & sig_jk = sig(*, j, k)
            hxi_jk = hxi_index(j, k)
            IF(KEYWORD_SET(calc_btot)) THEN btot_jk = btot0(j, k) $
              ELSE btot_jk = 0
            hxt_memimg, ictrl, hxi_jk, index1_k, fobs_jk, sig_jk, $
              ppp, index_jk, data_jk, bsmooth = bsmooth, $
              delta_max = delta_max, iter_max = iter_max, $
              lambda_limit = lambda_limit, lambda_max = lambda_max, $
              chi_limit = chilim, btot_in = btot_jk
; Load the output arrays
            dset = k*nch+j
            str_load, index_out, index = dset, index_jk
            data_out(*, *, dset) = data_jk
            imagedata(*, dset) = fobs_jk
            imagesig(*, dset) = sig_jk
         ENDFOR
      ENDFOR
      message, /info, 'Computing error estimate'
      error = hxt_error(index_out, imagedata, p, data_out, imagesig)
   ENDELSE

; Normalize the data, if not asked not to
   IF(NOT KEYWORD_SET(nonorm)) THEN data_out = hxi_normap(temporary(data_out))
; Output the data, if necessary
   IF(outfil NE 'none') THEN sav_hxi, outfil, index_out, data_out, append = append
      
   RETURN
END


